@@ -8,6 +8,9 @@
  * This file handles the architecture-dependent parts of process handling..
  */
 
+#define __KERNEL_SYSCALLS__
+#include <stdarg.h>
+
 #include <linux/errno.h>
 #include <linux/sched.h>
 #include <linux/kernel.h>
@@ -19,6 +22,9 @@
 #include <linux/ldt.h>
 #include <linux/user.h>
 #include <linux/a.out.h>
+#include <linux/interrupt.h>
+#include <linux/config.h>
+#include <linux/unistd.h>
 
 #include <asm/segment.h>
 #include <asm/pgtable.h>
@@ -29,8 +35,15 @@
 
 asmlinkage void ret_from_sys_call(void) __asm__("ret_from_sys_call");
 
+#ifdef CONFIG_APM
+extern int  apm_do_idle(void);
+extern void apm_do_busy(void);
+#endif
+
 static int hlt_counter=0;
 
+#define HARD_IDLE_TIMEOUT (HZ / 3)
+
 void disable_hlt(void)
 {
 	hlt_counter++;
@@ -41,51 +54,103 @@ void enable_hlt(void)
 	hlt_counter--;
 }
 
+#ifndef __SMP__
+
+static void hard_idle(void)
+{
+	while (!need_resched) {
+		if (hlt_works_ok && !hlt_counter) {
+#ifdef CONFIG_APM
+				/* If the APM BIOS is not enabled, or there
+				 is an error calling the idle routine, we
+				 should hlt if possible.  We need to check
+				 need_resched again because an interrupt
+				 may have occured in apm_do_idle(). */
+			start_bh_atomic();
+			if (!apm_do_idle() && !need_resched)
+				__asm__("hlt");
+			end_bh_atomic();
+#else
+			__asm__("hlt");
+#endif
+	        }
+ 		if (need_resched) 
+ 			break;
+		schedule();
+	}
+#ifdef CONFIG_APM
+	apm_do_busy();
+#endif
+}
+
 /*
- * The idle loop on a i386..
+ * The idle loop on a uniprocessor i386..
  */
+ 
 asmlinkage int sys_idle(void)
 {
+        unsigned long start_idle = 0;
+
 	if (current->pid != 0)
-	{
-	/*	printk("Wrong process idled\n");	SMP bug check */
 		return -EPERM;
-	}
-#ifdef __SMP__
-	/*
-	 *	SMP locking sanity checker
-	 */
-	if(smp_processor_id()!=active_kernel_processor)
-		panic("CPU is %d, kernel CPU is %d in sys_idle!\n",
-			smp_processor_id(), active_kernel_processor);
-	if(syscall_count!=1)
-		printk("sys_idle: syscall count is not 1 (%ld)\n", syscall_count);
-	if(kernel_counter!=1)
-	{
-		printk("CPU %d, sys_idle, kernel_counter is %ld\n", smp_processor_id(), kernel_counter);
-		if(!kernel_counter)
-			panic("kernel locking botch");
-	}
-	/*
-	 *	Until we have C unlocking done
-	 */
-	current->counter = -100;
-	schedule();
-	return 0;
-#endif	
 	/* endless idle loop with no priority at all */
 	current->counter = -100;
-	for (;;) {
-#ifdef __SMP__
-		if (cpu_data[smp_processor_id()].hlt_works_ok && !hlt_counter && !need_resched)
-#else	
-		if (hlt_works_ok && !hlt_counter && !need_resched)
-#endif		
-			__asm__("hlt");
+	for (;;) 
+	{
+		/*
+		 *	We are locked at this point. So we can safely call
+		 *	the APM bios knowing only one CPU at a time will do
+		 *	so.
+		 */
+		if (!start_idle) 
+			start_idle = jiffies;
+		if (jiffies - start_idle > HARD_IDLE_TIMEOUT) 
+		{
+			hard_idle();
+		} 
+		else 
+		{
+			if (hlt_works_ok && !hlt_counter && !need_resched)
+		        	__asm__("hlt");
+		}
+		if (need_resched) 
+			start_idle = 0;
 		schedule();
 	}
 }
 
+#else
+
+/*
+ *	In the SMP world we hlt outside of kernel syscall rather than within
+ *	so as to get the right locking semantics.
+ */
+ 
+asmlinkage int sys_idle(void)
+{
+	if(current->pid != 0)
+		return -EPERM;
+	current->counter= -100;
+	schedule();
+	return 0;
+}
+
+/*
+ *	This is being executed in task 0 'user space'.
+ */
+
+int cpu_idle(void *unused)
+{
+	while(1)
+	{
+		if(cpu_data[smp_processor_id()].hlt_works_ok && !hlt_counter && !need_resched)
+			__asm("hlt");
+		idle();
+	}
+}
+
+#endif
+
 /*
  * This routine reboots the machine by asking the keyboard
  * controller to pulse the reset-line low. We try that for a while,
@@ -140,6 +205,7 @@ void show_regs(struct pt_regs * regs)
 /*
  * Free current thread data structures etc..
  */
+
 void exit_thread(void)
 {
 	/* forget lazy i387 state */
