@@ -38,6 +38,12 @@
  *
  * Reorganized FASYNC support so mouse code can share it.
  *	-- ctm@ardi.com, 9Sep95
+ *
+ * New TIOCLINUX variants added.
+ *	-- mj@k332.feld.cvut.cz, 19-Nov-95
+ * 
+ * Restrict vt switching via ioctl()
+ *      -- grif@cs.ucr.edu, 5-Dec-95
  */
 
 #include <linux/config.h>
@@ -84,14 +90,17 @@ struct tty_ldisc ldiscs[NR_LDISCS];	/* line disc dispatch table	*/
 
 /*
  * fg_console is the current virtual console,
- * last_console is the last used one
+ * last_console is the last used one,
+ * kmsg_redirect is the console for kernel messages,
  * redirect is the pseudo-tty that console output
  * is redirected to if asked by TIOCCONS.
  */
 int fg_console = 0;
 int last_console = 0;
+int kmsg_redirect = 0;
 struct tty_struct * redirect = NULL;
 struct wait_queue * keypress_wait = NULL;
+char vt_dont_switch = 0;
 
 static void initialize_tty_struct(struct tty_struct *tty);
 
@@ -518,7 +527,7 @@ void complete_change_console(unsigned int new_console)
 {
 	unsigned char old_vc_mode;
 
-        if (new_console == fg_console)
+        if ((new_console == fg_console) || (vt_dont_switch))
                 return;
         if (!vc_cons_allocated(new_console))
                 return;
@@ -589,7 +598,7 @@ void complete_change_console(unsigned int new_console)
  */
 void change_console(unsigned int new_console)
 {
-        if (new_console == fg_console)
+        if ((new_console == fg_console) || (vt_dont_switch))
                 return;
         if (!vc_cons_allocated(new_console))
 		return;
@@ -1542,6 +1551,17 @@ static int tty_ioctl(struct inode * inode, struct file * file,
 				case 10:
 					set_vesa_blanking(arg);
 					return 0;
+				case 11:	/* set kmsg redirect */
+					if (!suser())
+						return -EPERM;
+					retval = verify_area(VERIFY_READ,
+						(void *) arg+1, 1);
+					if (retval)
+						return retval;
+					kmsg_redirect = get_user((char *)arg+1);
+					return 0;
+				case 12:	/* get fg_console */
+					return fg_console;
 				default: 
 					return -EINVAL;
 			}
