@@ -26,7 +26,6 @@
 #include <linux/sched.h>
 #include <linux/proc_fs.h>
 #include <linux/stat.h>
-#include <linux/config.h>
 #include <linux/mm.h>
 
 /* forward references */
@@ -71,6 +70,8 @@ struct inode_operations proc_scsi_inode_operations = {
     NULL,	    /* rename	   */
     NULL,	    /* readlink	   */
     NULL,	    /* follow_link */
+    NULL,	    /* readpage	   */
+    NULL,	    /* writepage   */
     NULL,	    /* bmap	   */
     NULL,	    /* truncate	   */
     NULL	    /* permission  */
