@@ -79,7 +79,7 @@ __asm__ __volatile__( \
 	"bis %0,%0,$16\n\t" \
 	"call_pal 53" \
 	: : "r" (__new_ipl) \
-	: "$0", "$1", "$16", "$22", "$23", "$24", "$25")
+	: "$0", "$1", "$16", "$22", "$23", "$24", "$25", "memory")
 
 #define swpipl(__new_ipl) \
 ({ unsigned long __old_ipl; \
@@ -89,7 +89,7 @@ __asm__ __volatile__( \
 	"bis $0,$0,%0" \
 	: "=r" (__old_ipl) \
 	: "r" (__new_ipl) \
-	: "$0", "$1", "$16", "$22", "$23", "$24", "$25"); \
+	: "$0", "$1", "$16", "$22", "$23", "$24", "$25", "memory"); \
 __old_ipl; })
 
 #define cli()			setipl(7)
@@ -97,18 +97,25 @@ __old_ipl; })
 #define save_flags(flags)	do { flags = getipl(); } while (0)
 #define restore_flags(flags)	setipl(flags)
 
+/*
+ * Give prototypes to shut up gcc.
+ */
+extern inline unsigned long xchg_u32 (volatile int * m, unsigned long val);
+extern inline unsigned long xchg_u64 (volatile long * m, unsigned long val);
+
 extern inline unsigned long xchg_u32(volatile int * m, unsigned long val)
 {
 	unsigned long dummy, dummy2;
 
 	__asm__ __volatile__(
 		"\n1:\t"
-		"ldl_l %0,%1\n\t"
+		"ldl_l %0,0(%1)\n\t"
 		"bis %2,%2,%3\n\t"
-		"stl_c %3,%1\n\t"
+		"stl_c %3,0(%1)\n\t"
 		"beq %3,1b\n"
-		: "=r" (val), "=m" (*m), "=r" (dummy), "=r" (dummy2)
-		: "1" (*m), "2" (val));
+		: "=r" (val), "=r" (m), "=r" (dummy), "=r" (dummy2)
+		: "1" (m), "2" (val)
+		: "memory");
 	return val;
 }
 
@@ -118,12 +125,13 @@ extern inline unsigned long xchg_u64(volatile long * m, unsigned long val)
 
 	__asm__ __volatile__(
 		"\n1:\t"
-		"ldq_l %0,%1\n\t"
+		"ldq_l %0,0(%1)\n\t"
 		"bis %2,%2,%3\n\t"
-		"stq_c %3,%1\n\t"
+		"stq_c %3,0(%1)\n\t"
 		"beq %3,1b\n"
-		: "=r" (val), "=m" (*m), "=r" (dummy), "=r" (dummy2)
-		: "1" (*m), "2" (val));
+		: "=r" (val), "=r" (m), "=r" (dummy), "=r" (dummy2)
+		: "1" (m), "2" (val)
+		: "memory");
 	return val;
 }
 
