@@ -46,7 +46,7 @@ swap_control_t swap_control = {
 	20, 3, 1, 3,		/* Page aging */
 	10, 2, 2, 0,		/* Buffer aging */
 	32, 4,			/* Aging cluster */
-	8192, 4096,		/* Pageout and bufferout weights */
+	8192, 8192,		/* Pageout and bufferout weights */
 	-200,			/* Buffer grace */
 	1, 1,			/* Buffs/pages to free */
 	RCL_ROUND_ROBIN		/* Balancing policy */
@@ -457,7 +457,7 @@ static inline int try_to_swap_out(struct task_struct * tsk, struct vm_area_struc
 				return 0;
 			vma->vm_mm->rss--;
 			set_pte(page_table, __pte(entry));
-			invalidate_page(vma->vm_mm, address);
+			invalidate_page(vma, address);
 			tsk->nswap++;
 			write_swap_page(entry, (char *) page);
 		}
@@ -472,14 +472,14 @@ static inline int try_to_swap_out(struct task_struct * tsk, struct vm_area_struc
 		}
 		vma->vm_mm->rss--;
 		set_pte(page_table, __pte(entry));
-		invalidate_page(vma->vm_mm, address);
+		invalidate_page(vma, address);
 		free_page(page);
 		return 1;
 	} 
 	vma->vm_mm->rss--;
 	pte_clear(page_table);
-	invalidate_page(vma->vm_mm, address);
-	entry = mem_map[MAP_NR(page)].count;
+	invalidate_page(vma, address);
+	entry = page_unuse(page);
 	free_page(page);
 	return entry;
 }
@@ -687,6 +687,10 @@ static int try_to_free_page(int priority, unsigned long limit)
 			if (shm_swap(i, limit))
 				return 1;
 			state = 2;
+		case 2:
+			if (shrink_mmap(i, limit))
+				return 1;
+			state = 3;
 		default:
 			if (swap_out(i, limit))
 				return 1;
@@ -1324,6 +1328,11 @@ unsigned long free_area_init(unsigned long start_mem, unsigned long end_mem)
 		p->count = 0;
 		p->dirty = 0;
 		p->reserved = 1;
+		p->inode = NULL;
+		p->offset = 0;
+		p->write_list = NULL;
+		p->next = p->prev = NULL;
+		p->next_hash = p->next_hash = NULL;
 	}
 
 	for (i = 0 ; i < NR_MEM_LISTS ; i++) {
